"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const uuid = require("uuid");
const const_1 = require("../const");
const ProcfileReconciler_1 = require("../application/ProcfileReconciler");
const ApplicationHandler_1 = require("../application/ApplicationHandler");
const fs_1 = require("mz/fs");
const LoggerBroker_1 = require("../universal/LoggerBroker");
class ComplexHandler {
    constructor(appRepresentation) {
        this.appId = null;
        this.appRepresentation = null;
        this.state = const_1.State.pending;
        this.appId = uuid.v4();
        this.appRepresentation = appRepresentation;
    }
    get name() {
        return this.appRepresentation.appName;
    }
    get appDir() {
        return this.appRepresentation.appDir;
    }
    get mode() {
        return this.appRepresentation.mode;
    }
    get pids() {
        const ret = [];
        if (this.mountedApplications) {
            for (const app of this.mountedApplications) {
                if (app.pid) {
                    ret.push(app.pid);
                }
            }
        }
        return ret;
    }
    get startCount() {
        let ret = 0;
        if (this.mountedApplications) {
            for (const app of this.mountedApplications) {
                ret += app.startCount;
            }
        }
        return ret;
    }
    get uptime() {
        if (!this.startTime) {
            return 0;
        }
        return (Date.now() - this.startTime) / 1000;
    }
    async getComplex() {
        if (!this.complexStructure) {
            this.complexStructure = await ProcfileReconciler_1.ProcfileReconciler.getComplexViaNewProcess(this.appRepresentation);
        }
        return this.complexStructure;
    }
    async fillMounted() {
        if (this.mountedApplications) {
            return;
        }
        this.mountedApplications = [];
        const { mount: mountList } = await this.getComplex();
        for (const mount of mountList) {
            const appHandler = new ApplicationHandler_1.ApplicationHandler(mount);
            this.mountedApplications.push(appHandler);
        }
    }
    async start() {
        if (!await fs_1.exists(this.appDir)) {
            new Error(`AppDir ${this.appDir} does not exist`);
        }
        await this.backupStdoutLogFile();
        await this.fillMounted();
        if (!this.mountedApplications.length) {
            throw new Error('Start failed, looks like not a pandora project, in appDir ' + this.appDir);
        }
        const startedHandlers = [];
        for (const appHandler of this.mountedApplications) {
            try {
                await appHandler.start();
            }
            catch (error) {
                for (const started of startedHandlers) {
                    await started.stop();
                }
                throw error;
            }
            startedHandlers.push(appHandler);
        }
        this.state = const_1.State.complete;
        this.startTime = Date.now();
    }
    async stop() {
        if (this.state === const_1.State.stopped) {
            return;
        }
        for (const appHandler of this.mountedApplications) {
            await appHandler.stop();
        }
        this.state = const_1.State.stopped;
    }
    async reload(processName) {
        for (const appHandler of this.mountedApplications) {
            await appHandler.reload(processName);
        }
    }
    async backupStdoutLogFile() {
        const targetPath = LoggerBroker_1.getAppLogPath(this.name, 'nodejs_stdout');
        if (await fs_1.exists(targetPath)) {
            await LoggerBroker_1.backupLog(targetPath);
        }
    }
}
exports.ComplexHandler = ComplexHandler;
//# sourceMappingURL=ComplexHandler.js.map